@@ -702,9 +702,9 @@ int __glXSwapRenderLarge(__GLXclientState *cl, GLbyte *pc)
 	*/
 	if (cl->largeCmdBufSize < hdr->length) {
 	    if (!cl->largeCmdBuf) {
-		cl->largeCmdBuf = (GLbyte *) __glXMalloc(hdr->length);
+		cl->largeCmdBuf = (GLbyte *) malloc(hdr->length);
 	    } else {
-		cl->largeCmdBuf = (GLbyte *) __glXRealloc(cl->largeCmdBuf, hdr->length);
+		cl->largeCmdBuf = (GLbyte *) realloc(cl->largeCmdBuf, hdr->length);
 	    }
 	    if (!cl->largeCmdBuf) {
 	       cl->largeCmdRequestsTotal = 0;
